// ScribDoc.cpp : implementation of the CScribbleDoc class
//
// This is a part of the Microsoft Foundation Classes C++ library.
// Copyright (C) 1992-1997 Microsoft Corporation
// All rights reserved.
//
// This source code is only intended as a supplement to the
// Microsoft Foundation Classes Reference and related
// electronic documentation provided with the library.
// See these sources for detailed information regarding the
// Microsoft Foundation Classes product.

#include "stdafx.h"
#include "Scribble.h"

#include "ScribDoc.h"
#include "ScribVw.h"		// @@ added line

#include "MainFrm.h"
#include "PenDlg.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CScribbleDoc

IMPLEMENT_DYNCREATE(CScribbleDoc, CDocument)

BEGIN_MESSAGE_MAP(CScribbleDoc, CDocument)
	//{{AFX_MSG_MAP(CScribbleDoc)
	ON_COMMAND(ID_EDIT_CLEAR_ALL, OnEditClearAll)
	ON_COMMAND(ID_PEN_THICK_OR_THIN, OnPenThickOrThin)
	ON_UPDATE_COMMAND_UI(ID_EDIT_CLEAR_ALL, OnUpdateEditClearAll)
	ON_UPDATE_COMMAND_UI(ID_PEN_THICK_OR_THIN, OnUpdatePenThickOrThin)
	ON_COMMAND(ID_PEN_WIDTHS, OnPenWidths)
	ON_UPDATE_COMMAND_UI(ID_EDIT_UNDO, OnUpdateEditUndo)
	ON_COMMAND(ID_EDIT_UNDO, OnEditUndo)
	ON_UPDATE_COMMAND_UI(ID_EDIT_REDO, OnUpdateEditRedo)
	ON_COMMAND(ID_EDIT_REDO, OnEditRedo)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CScribbleDoc construction/destruction

CScribbleDoc::CScribbleDoc()
{
	// TODO: add one-time construction code here

}

CScribbleDoc::~CScribbleDoc()
{
}

BOOL CScribbleDoc::OnNewDocument()
{
	if (!CDocument::OnNewDocument())
		return FALSE;
	InitDocument();
	return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// CScribbleDoc serialization

void CScribbleDoc::Serialize(CArchive& ar)
{
	if (ar.IsStoring())
	{
	}
	else
	{
	}
	m_strokeList.Serialize(ar);
}

/////////////////////////////////////////////////////////////////////////////
// CScribbleDoc diagnostics

#ifdef _DEBUG
void CScribbleDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CScribbleDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CScribbleDoc commands

BOOL CScribbleDoc::OnOpenDocument(LPCTSTR lpszPathName) 
{
	if (!CDocument::OnOpenDocument(lpszPathName))
		return FALSE;
	InitDocument(); 
	return TRUE;
}

void CScribbleDoc::DeleteContents() 
{
	ResetHistory();							// @@ added line
	m_strokeList.RemoveAll();				// @@ added line

/* @@ deleted block
	while (!m_strokeList.IsEmpty())
	{
		delete m_strokeList.RemoveHead();
	}
*/
	CDocument::DeleteContents();
}

void CScribbleDoc::InitDocument()
{
	m_bThickPen = FALSE;
	m_nThinWidth = 2;   // default thin pen is 2 pixels wide
	m_nThickWidth = 5;  // default thick pen is 5 pixels wide
	ReplacePen();       // initialize pen according to current width
}

/* @@ deleted member definition
CStroke* CScribbleDoc::NewStroke()
{
	CStroke* pStrokeItem = new CStroke(m_nPenWidth);
	m_strokeList.AddTail(pStrokeItem);
	SetModifiedFlag();  // Mark the document as having been modified, for
						// purposes of confirming File Close.
	return pStrokeItem;
}
*/

// @@ added member definition
BOOL CScribbleDoc::AddStroke(CStroke* pStrokeItem)
{
	TRY
	{
		m_strokeList.AddTail(pStrokeItem);
	}
	CATCH_ALL(e)
	{
		e->Delete();
		return FALSE;
	}
	END_CATCH_ALL

	return TRUE;
}

// @@ added member definition
BOOL CScribbleDoc::AddStrokes(const CStrokeList& list)
{
	DWORD dwCount = 0;

	TRY
	{
		for (POSITION pos = list.GetHeadPosition(); pos != NULL; )
		{
			CStroke* pStroke = list.GetNext(pos);
			if (m_strokeList.Find(pStroke) == NULL)	// no dups
			{
				m_strokeList.AddTail(pStroke);
				dwCount++;
			}
		}
	}
	CATCH_ALL(e)
	{
		e->Delete();
	}
	END_CATCH_ALL

	return dwCount;		// Returns success even if only one was successfully added
}

// @@ added member definition
BOOL CScribbleDoc::RemoveStroke(CStroke* pStrokeItem)
{
	POSITION pos;

	TRY
	{
		pos = m_strokeList.Find(pStrokeItem);
		if (!pos)
			return FALSE;
		m_strokeList.RemoveAt(pos);
	}
	CATCH_ALL(e)
	{
		e->Delete();
		return FALSE;
	}
	END_CATCH_ALL

	return TRUE;
}

// @@ added member definition
BOOL CScribbleDoc::RemoveStrokes(const CStrokeList& list)
{
	DWORD dwCount = 0;
	TRY
	{
		for (POSITION posOut = list.GetHeadPosition(); posOut != NULL; )
		{
			CStroke* pStroke = list.GetNext(posOut);
			POSITION posIn = m_strokeList.Find(pStroke);
			if (posIn)
			{
				m_strokeList.RemoveAt(posIn);
				dwCount++;
			}
		}
	}
	CATCH_ALL(e)
	{
		e->Delete();
	}
	END_CATCH_ALL

	return dwCount;		// Returns success even if only one was successfully removed
}




/////////////////////////////////////////////////////////////////////////////
// CStroke

IMPLEMENT_SERIAL(CStroke, CRefObject, 1)	// @@ changed base from CObject to CRefObject
CStroke::CStroke()
{
	// This empty constructor should be used by serialization only
}

CStroke::CStroke(UINT nPenWidth)
{
	m_nPenWidth = nPenWidth;
}

void CStroke::Serialize(CArchive& ar)
{
	if (ar.IsStoring())
	{
		ar << (WORD)m_nPenWidth;
		m_pointArray.Serialize(ar);
	}
	else
	{
		WORD w;
		ar >> w;
		m_nPenWidth = w;
		m_pointArray.Serialize(ar);
	}
}

BOOL CStroke::DrawStroke(CDC* pDC)
{
	CPen penStroke;
	if (!penStroke.CreatePen(PS_SOLID, m_nPenWidth, RGB(0,0,0)))
		return FALSE;
	CPen* pOldPen = pDC->SelectObject(&penStroke);
	pDC->MoveTo(m_pointArray[0]);
	for (int i=1; i < m_pointArray.GetSize(); i++)
	{
		pDC->LineTo(m_pointArray[i]);
	}

	pDC->SelectObject(pOldPen);
	return TRUE;
}

// @@ added member definition
CRect CStroke::GetBoundingRect() const
{
	CRect rect(2147483647,  2147483647,  -2147483647, -2147483647);

	for (int i=0; i< m_pointArray.GetSize(); i++)
	{
		if (m_pointArray[i].x < rect.left)
			rect.left = m_pointArray[i].x;
		if (m_pointArray[i].x > rect.right)
			rect.right = m_pointArray[i].x;

		if (m_pointArray[i].y < rect.top)
			rect.top = m_pointArray[i].y;
		if (m_pointArray[i].y > rect.bottom)
			rect.bottom = m_pointArray[i].y;
	}
	rect.InflateRect(1, 1, 1, 1);
	return rect;
}


void CScribbleDoc::OnUpdateEditClearAll(CCmdUI* pCmdUI) 
{
	// Enable the command user interface object (menu item or tool bar
	// button) if the document is non-empty, i.e., has at least one stroke.
	pCmdUI->Enable(!m_strokeList.IsEmpty());
}

void CScribbleDoc::OnEditClearAll() 
{
/* @@ deleted block
	DeleteContents();
	SetModifiedFlag();  // Mark the document as having been modified, for
						// purposes of confirming File Close.
	UpdateAllViews(NULL);
*/
	Do(new CCmdRemoveStrokes(this, m_strokeList), TRUE);		// @@ added line
}

void CScribbleDoc::OnUpdatePenThickOrThin(CCmdUI* pCmdUI) 
{
	// Add check mark to Draw Thick Line menu item, if the current
	// pen width is "thick".
	pCmdUI->SetCheck(m_bThickPen);
}

void CScribbleDoc::OnPenThickOrThin() 
{
	// Toggle the state of the pen between thin or thick.
	m_bThickPen = !m_bThickPen;

	// Change the current pen to reflect the new user-specified width.
	ReplacePen();
}

void CScribbleDoc::ReplacePen()
{
	m_nPenWidth = m_bThickPen? m_nThickWidth : m_nThinWidth;

	// Change the current pen to reflect the new user-specified width.
	m_penCur.DeleteObject();
	m_penCur.CreatePen(PS_SOLID, m_nPenWidth, RGB(0,0,0)); // solid black
}

void CScribbleDoc::OnPenWidths() 
{
	CPenWidthsDlg dlg;
	// Initialize dialog data
	dlg.m_nThinWidth = m_nThinWidth;
	dlg.m_nThickWidth = m_nThickWidth;

	// Invoke the dialog box
	if (dlg.DoModal() == IDOK)
	{
		// retrieve the dialog data
		m_nThinWidth = dlg.m_nThinWidth;
		m_nThickWidth = dlg.m_nThickWidth;

		// Update the pen that is used by views when drawing new strokes,
		// to reflect the new pen width definitions for "thick" and "thin".
		ReplacePen();
	}
}

// @@ added member
void CScribbleDoc::OnUpdateEditUndo(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(CanUndo());
}

// @@ added member
void CScribbleDoc::OnEditUndo() 
{
	Undo();
}

// @@ added member
void CScribbleDoc::OnUpdateEditRedo(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(CanRedo());
}

// @@ added member
void CScribbleDoc::OnEditRedo() 
{
	Redo();
}


/////////////////////////////////////////////////////////////////////////////
// USING THE COMMAND HISTORY "CLEAN" STATE
/////////////////////////////////////////////////////////////////////////////

// @@ added override (optional as long as you never call this member)
void CScribbleDoc::SetModifiedFlag(BOOL)
{
	// Do nothing
}

// @@ added override
BOOL CScribbleDoc::IsModified()
{
	return IsDirty();
}

// @@ added override
BOOL CScribbleDoc::OnSaveDocument(LPCTSTR lpszPathName) 
{
	if (CDocument::OnSaveDocument(lpszPathName))
	{
		SetClean();		// Tells the command history that the document is "clean" at this point
		return TRUE;
	}
	return FALSE;
}


/////////////////////////////////////////////////////////////////////////////
// NOTIFYING THE USER OF THE DOCUMENT'S "CLEAN" STATE
/////////////////////////////////////////////////////////////////////////////

// @@ added override
void CScribbleDoc::SetTitle(LPCTSTR lpszTitle)
{
	m_strTitleOrg = lpszTitle;
	CDocument::SetTitle(lpszTitle);
}

// @@ added member
void CScribbleDoc::UpdateTitle()
{
	if (m_strTitleOrg.IsEmpty() && !m_strTitle.IsEmpty())
		m_strTitleOrg = m_strTitle;

	if (!m_strTitleOrg.IsEmpty())
	{
		TCHAR szStr[256+_MAX_PATH];
		lstrcpy(szStr, m_strTitleOrg);
		if (IsDirty())
			lstrcat(szStr, _T(" *"));
		if (m_strTitle != szStr)
		{
			m_strTitle = szStr;
			UpdateFrameCounts();
		}
	}
}

// @@ added override
void ICustomCommandHistory::OnStatusChange()
{
	static_cast<CScribbleDoc*>(this)->UpdateTitle();

	// Display command history diagnostics on the status bar
	CString str;
	str.Format(_T("  P=%03d, U=%03d, R=%03d, C=%03d  "), m_iCommandHistoryUndoPos, m_iCommandHistoryUndoCount, m_iCommandHistoryRedoCount, m_iCommandHistoryCleanCount);
	STATIC_DOWNCAST(CMainFrame, AfxGetMainWnd())->SetPaneText(1, str);
}

void ICustomCommandHistory::OnCommandExecute(const CCommand* pCmd)
{
	CString strLabel;
	pCmd->Sprint(strLabel);
	CString str;
	str.Format(_T("Last command: %s"), strLabel);
	STATIC_DOWNCAST(CFrameWnd, AfxGetMainWnd())->SetMessageText(str);

	static_cast<CScribbleDoc*>(this)->UpdateAllViews(NULL, 0, STATIC_DOWNCAST(CObject, pCmd));
}
