// ScribVw.cpp : implementation of the CScribbleView class
//
// This is a part of the Microsoft Foundation Classes C++ library.
// Copyright (C) 1992-1997 Microsoft Corporation
// All rights reserved.
//
// This source code is only intended as a supplement to the
// Microsoft Foundation Classes Reference and related
// electronic documentation provided with the library.
// See these sources for detailed information regarding the
// Microsoft Foundation Classes product.

#include "stdafx.h"
#include "Scribble.h"

#include "ScribDoc.h"
#include "ScribVw.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CScribbleView

IMPLEMENT_DYNCREATE(CScribbleView, CView)

BEGIN_MESSAGE_MAP(CScribbleView, CView)
	//{{AFX_MSG_MAP(CScribbleView)
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_WM_MOUSEMOVE()
	//}}AFX_MSG_MAP
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CView::OnFilePrintPreview)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CScribbleView construction/destruction

CScribbleView::CScribbleView()
{
	// TODO: add construction code here

}

CScribbleView::~CScribbleView()
{
}

BOOL CScribbleView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CScribbleView drawing

void CScribbleView::OnDraw(CDC* pDC)
{
	CScribbleDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	// The view delegates the drawing of individual strokes to
	// CStroke::DrawStroke().
	const CStrokeList& strokeList = pDoc->m_strokeList;	// @@ changed strokeList type
	POSITION pos = strokeList.GetHeadPosition();
	while (pos != NULL)
	{
		CStroke* pStroke = strokeList.GetNext(pos);
		pStroke->DrawStroke(pDC);
	}
}

/////////////////////////////////////////////////////////////////////////////
// CScribbleView printing

BOOL CScribbleView::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	return DoPreparePrinting(pInfo);
}

void CScribbleView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add extra initialization before printing
}

void CScribbleView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add cleanup after printing
}

/////////////////////////////////////////////////////////////////////////////
// CScribbleView diagnostics

#ifdef _DEBUG
void CScribbleView::AssertValid() const
{
	CView::AssertValid();
}

void CScribbleView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CScribbleDoc* CScribbleView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CScribbleDoc)));
	return (CScribbleDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CScribbleView message handlers

void CScribbleView::OnLButtonDown(UINT, CPoint point) 
{
    // Pressing the mouse button in the view window starts a new stroke

//	m_pStrokeCur = GetDocument()->NewStroke();						// @@ deleted line
	m_pStrokeCur = new CStroke(GetDocument()->GetPenWidth());	// @@ added line
	m_pStrokeCur->AddRef();													// @@ added line

	// Add first point to the new stroke
	m_pStrokeCur->m_pointArray.Add(point);

	SetCapture();       // Capture the mouse until button up.
	m_ptPrev = point;   // Serves as the MoveTo() anchor point for the
						// LineTo() the next point, as the user drags the
						// mouse.

	return;
}

void CScribbleView::OnLButtonUp(UINT, CPoint point) 
{
	// Mouse button up is interesting in the Scribble application
	// only if the user is currently drawing a new stroke by dragging
	// the captured mouse.

	if (GetCapture() != this)
		return; // If this window (view) didn't capture the mouse,
				// then the user isn't drawing in this window.

	CScribbleDoc* pDoc = GetDocument();

	CClientDC dc(this);

	CPen* pOldPen = dc.SelectObject(pDoc->GetCurrentPen());
	dc.MoveTo(m_ptPrev);
	dc.LineTo(point);
	dc.SelectObject(pOldPen);
	m_pStrokeCur->m_pointArray.Add(point);

	ReleaseCapture();   // Release the mouse capture established at
						// the beginning of the mouse drag.

	{ // @@ added block
	pDoc->Do(new CCmdAddStroke(pDoc, m_pStrokeCur), TRUE);	// Do add stroke command
	m_pStrokeCur->Release();											// We no longer need the stroke object
	m_pStrokeCur = NULL;													// so we release it and set our pointer to NULL
	// The stroke still exists with two references: 1. by the document (in it's list)
	// and 2. by the CCmdAddStroke command who was passed down to the command history
	// The stroke will be deleted when A) the document releases it (CScribbleDoc::DeleteHistory)
	// AND B) the command is deleted by the command history (CCmdAddStroke::~CCmdAddStroke).  This
	// will occur either when the command "falls off" the ring buffer, or the buffer is destroyed.
	}

	return;
}

void CScribbleView::OnMouseMove(UINT, CPoint point) 
{
	// Mouse movement is interesting in the Scribble application
	// only if the user is currently drawing a new stroke by dragging
	// the captured mouse.

	if (GetCapture() != this)
		return; // If this window (view) didn't capture the mouse,
				// then the user isn't drawing in this window.

	CClientDC dc(this);
	m_pStrokeCur->m_pointArray.Add(point);

	// Draw a line from the previous detected point in the mouse
	// drag to the current point.
	CPen* pOldPen = dc.SelectObject(GetDocument()->GetCurrentPen());
	dc.MoveTo(m_ptPrev);
	dc.LineTo(point);
	dc.SelectObject(pOldPen);
	m_ptPrev = point;
	return;
}

void CScribbleView::OnUpdate(CView*, LPARAM, CObject* pHint) 
{
	if (pHint && pHint->IsKindOf(RUNTIME_CLASS(CCmdScribble)))
	{
		CCmdScribble* pCommand = STATIC_DOWNCAST(CCmdScribble, pHint);
		switch (COMMAND_ID(pCommand))
		{
			case CMD_ADDSTROKE:
			{
				CStroke* pStroke = STATIC_DOWNCAST(CCmdAddStroke, pCommand)->m_pStroke;
				CClientDC dc(this);
				pStroke->DrawStroke(&dc);
				break;
			}
			case CMD_ADDSTROKES:
			{
				const CStrokeList& rList = STATIC_DOWNCAST(CCmdAddStrokes, pCommand)->m_list;
				CClientDC dc(this);
				for (POSITION pos = rList.GetHeadPosition(); pos != NULL; )
					rList.GetNext(pos)->DrawStroke(&dc);
				break;
			}
			case CMD_REMOVESTROKE:
			{
				CStroke* pStroke = STATIC_DOWNCAST(CCmdRemoveStroke, pCommand)->m_pStroke;
				InvalidateRect(pStroke->GetBoundingRect());
				break;
			}
			case CMD_REMOVESTROKES:
			{
				const CStrokeList& rList = STATIC_DOWNCAST(CCmdRemoveStrokes, pCommand)->m_list;
				for (POSITION pos = rList.GetHeadPosition(); pos != NULL; )
					InvalidateRect(rList.GetNext(pos)->GetBoundingRect());
				break;
			}
		}
	}
}


IMPLEMENT_COMMAND_NOID(CCmdScribble, CCommand)

IMPLEMENT_COMMAND(CCmdAddStroke, CCmdScribble, CMD_ADDSTROKE)

BOOL CCmdAddStroke::Execute()
{
	ASSERT_VALID(m_pStroke);
	return m_pDoc->AddStroke(m_pStroke);
}

CCommand* CCmdAddStroke::GetUndoCommand() const
{
	return new CCmdRemoveStroke(m_pDoc, m_pStroke);
}

CCommand* CCmdAddStroke::GetRedoCommand() const
{
	return new CCmdAddStroke(m_pDoc, m_pStroke);
}


IMPLEMENT_COMMAND(CCmdRemoveStroke, CCmdScribble, CMD_REMOVESTROKE)

BOOL CCmdRemoveStroke::Execute()
{
	ASSERT_VALID(m_pStroke);
	return m_pDoc->RemoveStroke(m_pStroke);
}

CCommand* CCmdRemoveStroke::GetUndoCommand() const
{
	return new CCmdAddStroke(m_pDoc, m_pStroke);
}

CCommand* CCmdRemoveStroke::GetRedoCommand() const
{
	return new CCmdRemoveStroke(m_pDoc, m_pStroke);
}


IMPLEMENT_COMMAND(CCmdAddStrokes, CCmdScribble, CMD_ADDSTROKES)

CCmdAddStrokes::CCmdAddStrokes(CScribbleDoc* pDoc, const CStrokeList& list)
 : CCmdScribble(pDoc), m_list(list.GetCount())
{
	m_list.AddTail((CStrokeList*)&list);
}

BOOL CCmdAddStrokes::Execute()
{
	return m_pDoc->AddStrokes(m_list);
}

CCommand* CCmdAddStrokes::GetUndoCommand() const
{
	return new CCmdRemoveStrokes(m_pDoc, m_list);
}

CCommand* CCmdAddStrokes::GetRedoCommand() const
{
	return new CCmdAddStrokes(m_pDoc, m_list);
}


IMPLEMENT_COMMAND(CCmdRemoveStrokes, CCmdScribble, CMD_REMOVESTROKES)

CCmdRemoveStrokes::CCmdRemoveStrokes(CScribbleDoc* pDoc, const CStrokeList& list)
 : CCmdScribble(pDoc), m_list(list.GetCount())
{
	m_list.AddTail((CStrokeList*)&list);
}

BOOL CCmdRemoveStrokes::Execute()
{
	return m_pDoc->RemoveStrokes(m_list);
}

CCommand* CCmdRemoveStrokes::GetUndoCommand() const
{
	return new CCmdAddStrokes(m_pDoc, m_list);
}

CCommand* CCmdRemoveStrokes::GetRedoCommand() const
{
	return new CCmdRemoveStrokes(m_pDoc, m_list);
}
